"""Constants for output management."""

# Default configuration
DEFAULT_BASE_DIR: str = ".lintro"
DEFAULT_KEEP_LAST: int = 10
DEFAULT_TIMESTAMP_FORMAT: str = "%Y%m%d-%H%M%S"
DEFAULT_RUN_PREFIX: str = "run-"
DEFAULT_TEMP_PREFIX: str = ".lintro"
